@@ -730,11 +730,13 @@ class Client(Connection, tcp._BaseBaseClient):
         s = '<%s to %s at %x>' % (self.__class__, self.addr, unsignedID(self))
         return s
 
-
-    def __init__(self, host, port, bindAddress, connector, reactor=None):
+    def __init__(self, host, port, bindAddress, connector, reactor=None,
+                 timeout=-1):
         # BaseClient.__init__ is invoked later
         self.connector = connector
         self.addr = (host, port)
+        self.timeout = timeout
+        self.reactor = reactor
 
         whenDone = self.resolveAddress
         err = None
@@ -771,11 +773,12 @@ class Client(Connection, tcp._BaseBaseClient):
             # was scheduled via a callLater in self._finishInit
             return
 
-        # err = self.socket.getsockopt(udt.UDT_STATE)
-        # if err == udt.UDTSTATUS_CONNECTED:
-        #     self.failIfNotConnected(error.getConnectError((
-        #                 err, err)))
-        #     return
+        status = self.socket.getsockopt(udt.UDT_STATE)
+        if (status != udt.UDTSTATUS_CONNECTED) and \
+                (status != udt.UDTSTATUS_INIT):
+            self.failIfNotConnected(error.getConnectError((
+                        staus)))
+            return
 
         # doConnect gets called twice.  The first time we actually need to
         # start the connection attempt.  The second time we don't really
@@ -785,8 +788,6 @@ class Client(Connection, tcp._BaseBaseClient):
         # is not /particularly/ detrimental to do so.  This should get
         # cleaned up some day, though.
         try:
-            #FIXME connection happens in background,
-            #epoll monitoring needed
             connectResult = self.socket.connect_ex(self.realAddress)
         except udt.UDTException as ue:
             connectResult = ue[0]
@@ -802,15 +803,30 @@ class Client(Connection, tcp._BaseBaseClient):
                             connectResult, connectResult)))
                 return
 
-        # If I have reached this point without raising or returning, that means
-        # that the socket is connected.
-        del self.doWrite
-        del self.doRead
-        # we first stop and then start, to reset
-        #any references to the old doRead
-        self.stopReading()
-        self.stopWriting()
-        self._connectDone()
+
+
+        status = self.socket.getsockopt(udt.UDT_STATE)
+        if status == udt.UDTSTATUS_CONNECTED:
+            # If I have reached this point without raising
+            #or returning, that means
+            # that the socket is connected.
+            del self.doWrite
+            del self.doRead
+            # we first stop and then start, to reset
+            #any references to the old doRead
+            self.startReading()
+            self.startWriting()
+            self._connectDone()
+        else:
+            # If I have reached this point without raising
+            #or returning, that means
+            # that the socket is connected.
+            del self.doWrite
+            del self.doRead
+            # we first stop and then start, to reset
+            #any references to the old doRead
+            #self.startReading()
+            self.startWriting()
 
     def _connectDone(self):
         """
@@ -828,6 +844,23 @@ class Client(Connection, tcp._BaseBaseClient):
         self.startReading()
         self.protocol.makeConnection(self)
 
+    def doWrite(self):
+        try:
+            state = self.socket.getsockopt(udt.UDT_STATE)
+        except udt.UDTException as ue:
+            self.connected = False
+            self.failIfNotConnected(ue)
+        else:
+            if (state == udt.UDTSTATUS_CONNECTED) and (self.connected == False):
+                self._connectDone()
+                self.stopReading()
+                self.stopWriting()
+                self.startReading()
+                self.startWriting()
+                return
+            Connection.doWrite(self)
+
+
 class Connector(base.BaseConnector):
     """
     A L{Connector} provides of L{twisted.internet.interfaces.IConnector} for
@@ -843,6 +876,7 @@ class Connector(base.BaseConnector):
     def __init__(self, host, port, factory, timeout, bindAddress, reactor=None):
         self.host, self.port = host, port
         self.bindAddress = bindAddress
+        self.timeout = timeout
         base.BaseConnector.__init__(self, factory, timeout, reactor)
 
 
@@ -854,7 +888,7 @@ class Connector(base.BaseConnector):
         @rtype: L{Client}
         """
         return Client(self.host, self.port,
-                      self.bindAddress, self, self.reactor)
+                      self.bindAddress, self, self.reactor, self.timeout)
 
 
     def getDestination(self):
